/* 
 * File:   Analy.cpp
 * Author: xyang
 * 
 * Created on December 7, 2009, 9:26 AM
 */

#include "Analy.h"

Analy::Analy() {
}

Analy::Analy(const Analy& orig) {
}

Analy::~Analy() {
}

void clear(int& num, int& pos, cvec_t& S, ivec_t& SL, cvec_t& Q, ivec_t& QL) {
    num = 0;
    pos = 0;
    S.clear();
    SL.clear();
    Q.clear();
    QL.clear();
}

void Analy::run(const Para& myPara) {

    bIO::FASTA_input fasta_sr(myPara.iFaName);
    bIO::FASTA_input fasta_q(myPara.iQName);

    if (fasta_sr.operator bool() == false) {
        std::cout << "open " << myPara.iFaName << "failed, does it exist? \n";
        exit(1);
    }

    std::cout << "Run with Batch size: " << myPara.batchSize << "\n\t Batch:";
    int batch = 0;

    double timing = get_time();
    
    while (fasta_sr.operator bool()) {

        std::cout << batch << "\t";
        batch++;

        Seq mySeq;
        Kmer tmpKArray;

        mySeq.retrieve_seqs_from_fa(fasta_sr, myPara.batchSize, false);
        if (myPara.IFlag) {
            mySeq.retrieve_seqs_from_fa(fasta_q, myPara.batchSize, true);

            if (myPara.qFlag) {
                mySeq.qualHist(myPara);
            }
        }

        tmpKArray.getKC(mySeq, myPara);
        mergeKInfo(mySeq, KArray_, tmpKArray.getElem(), myPara.qFlag);

        print_time("", timing);

    }
    std::cout << "done!\n";

    // quality Histogram
    if (myPara.qFlag) {

        std::cout << "deriving QualHistogram\n";

        std::ofstream oHandle(myPara.oQHistName.c_str());
        if (oHandle.good())
            oHandle << "\nQuality Score Histogram\n\n";

        prefix_sum(q2cnt_, oHandle);
        oHandle.close();

        //std::cout << "For MaxER = " << myPara.maxER
        //          << " qualThreshold should be set to " << newThreshold << "\n";

        std::cout << "\t Done!\n";
    }

    // kmer cardinality histogram
    kmerHist(myPara.oKHistName, myPara.qualThreshold);

    // disable the output of kid-->cnt to file
    //printKID2Cnt(myPara);
}

void Analy::kmerHist(const std::string& filename, int Qthreshold) {
    std::cout << "deriving Kmer Cnt histogram:\n";
    iu64map_t good2Cnt, all2Cnt; // good kmers and all kmers
    for (unsigned int i = 0; i < KArray_.size(); ++ i){
        int goodCnt = KArray_[i].goodCnt;
        int cnt = KArray_[i].cnt;
        iu64map_t::iterator it(good2Cnt.find(goodCnt));
        if (it != good2Cnt.end()) {
            (*it).second++;
        } else {
            good2Cnt[goodCnt] = 1;
        }
        it = all2Cnt.find(cnt);
        if (it != all2Cnt.end()) {
            (*it).second++;
        } else {
            all2Cnt[cnt] = 1;
        }
    }

    std::ofstream oHandle(filename.c_str());
    if (oHandle.good())
        oHandle << "\n Kmer Count Histogram\n\n";
    prefix_sum(all2Cnt, oHandle);

    oHandle << "\n Good Kmer Count Histogram (Q = " << Qthreshold << " )\n\n";
    prefix_sum(good2Cnt, oHandle);

    oHandle.close();

    std::cout << "\tdone!!\n";
}

/*
 * Merge <ID, good_cnt, cnt> tuples
 */
void Analy::mergeKInfo(const Seq& mySeq, kcvec_t& TargetArray,
                       const kcvec_t& sourceArray, int qFlag){

    // quality histogram
    if (qFlag) {
        iu64map_t::const_iterator it_s2c(mySeq.getS2C().begin());
        for (; it_s2c != mySeq.getS2C().end(); ++it_s2c) {
            iu64map_t::iterator it_q2c = q2cnt_.find((*it_s2c).first);
            if (it_q2c != q2cnt_.end()) {
                (*it_q2c).second += (*it_s2c).second;
            } else {
                q2cnt_.insert(*it_s2c);
            }
        }
    }

    // Merging Array
    if (TargetArray.size() == 0){
        TargetArray = sourceArray;
        return;
    }

    uint64_t oriSize = TargetArray.size();
    /*
     * for every item in sourceArray, identify its position in TargetArray
     * Once identified, merge. Otherwise, add to the end of TargetArray to
     * be processed later
     */
    for (unsigned int i = 0; i < sourceArray.size(); ++ i){
        std::pair<kcvec_t::iterator, kcvec_t::iterator> bounds =
        std::equal_range (TargetArray.begin(), TargetArray.begin() + oriSize,
                sourceArray[i], Knumcomp());
        if (bounds.first == bounds.second) { // didn't found
            TargetArray.push_back(sourceArray[i]);
        }
        else{
            (*bounds.first).cnt += sourceArray[i].cnt;
            (*bounds.first).goodCnt += sourceArray[i].goodCnt;
        }
    }
    std::sort(TargetArray.begin(), TargetArray.end(), Knumcomp());
}


 
void Analy::printKID2Cnt(const Para& myPara) {

    std::cout << "output kmer 2 cnt to " << myPara.oKCntName << "\t";

    std::ofstream output(myPara.oKCntName.c_str());

    if (!output.good()) {
        std::cout << "cant' write to file: " << myPara.oKCntName << std::endl;
        exit(1);
    }
    for (unsigned int i = 0; i < KArray_.size(); ++ i){
        output << KArray_[i].ID  << "\t";
        if (myPara.IFlag) {
            output << KArray_[i].goodCnt << "\t";
        }
        output << KArray_[i].cnt << "\n";
    }
    output.close();
    std::cout << "done\n";
}


/*
int bisearch(const kcvec_t& KArray, int size, uint64_t elem) {
    int lb = 0, ub = size - 1, mid;
    while (lb <= ub) {
        mid = (lb + ub) / 2;
        if (KArray[mid].ID == elem)
            return mid;
        else if (KArray[mid].ID < elem)
            lb = mid + 1;
        else if (KArray[mid].ID > elem)
            ub = mid - 1;
    }
    return -1;
}
*/