/* 
 * File:   util.h
 * Author: lx
 *
 * Created on December 5, 2009, 9:06 PM
 */

#ifndef _UTIL_H
#define	_UTIL_H

#include <iostream>
#include <fstream>
#include <map>
#include <vector>
#include <set>
#include <complex> //pow
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <stdint.h>

#if defined (_MSC_VER)
  #include "getWinTime.hpp"
#else
  #include <sys/time.h>
#endif 

typedef std::vector<std::string> strvec_t;
typedef std::map<int, int> imap_t;
typedef std::map<int, uint64_t> iu64map_t;
typedef std::set<int> iset_t;
typedef std::vector<int> ivec_t;
typedef std::vector<char> cvec_t;
typedef std::pair<int, int> ipair_t;

class Para {
public:
    std::string iFaName;
    std::string oKHistName;
    std::string oQHistName;
    std::string oKCntName;
    std::string iQName;
    int batchSize;
    int IFlag;          // 1: short reads file   0: genome file
    int K;
    int maxBadQPerKmer;
    int qualThreshold;
    int qFlag;          // 1: derive Q Histogram 0: don't 
    double maxER;

    Para(int argc, char** argv) : argnum(argc), arg(argv) {
        qualThreshold = 0;
        setPara();
    };

private:
    int argnum;
    char** arg;

    void setPara() {

        if (argnum != 2) { std::cout << "Specify Config file \n"; exit(1); }

        std::string line, s1;
        std::ifstream input(arg[1]);
        if (!input) { std::cout << "cant open " << arg[1] << "\n"; exit(1);}

        std::istringstream buf;

        while(getline(input, line)){
            buf.clear();
            buf.str(line);

            if (buf >> s1){
                if (s1 == "InFaFile")
                    buf >> iFaName;
                else if (s1 == "IFlag")
                    buf >> IFlag;
                else if (s1 == "BatchSize")
                    buf >> batchSize;
                else if (s1 == "KmerLen")
                    buf >> K;
                else if (s1 =="OKmerHistFile")
                    buf >> oKHistName;
                else if (s1 == "QHistFile")
                    buf >> oQHistName;
                else if (s1 == "OKmerCntFile")
                    buf >> oKCntName;
                else if (s1 == "IQFile")
                    buf >> iQName;
                else if (s1 == "MaxErrRate")
                    buf >> maxER;
                else if (s1 == "QThreshold")
                    buf >> qualThreshold;
                else if (s1 == "QFlag")
                    buf >> qFlag;
                else if (s1 == "MaxBadQPerKmer")
                    buf >> maxBadQPerKmer;
            }
        }

        std::cout << "Input Parameters:\n----------------------------------\n";
        std::cout << "Working with " << iFaName;
        if (IFlag) std::cout << " -- short reads file\n";
        else std::cout << " -- genome file\n";
        std::cout << "K = " << K << "\n"
                << "BatchSize = " << batchSize << "\n"
                << "O/KmerHistFile = " << oKHistName << "\n"
                << "O/Kmer2CntFile = " << oKCntName << "\n";
        if (IFlag) {
            std::cout << "I/QualFile = " << iQName << "\n"
                    << "MaxErrRate = " << maxER << "\n"
                    << "Qthreshold = " << qualThreshold << "\n"
                    << "MaxBadQ/Kmer = " << maxBadQPerKmer << "\n"
                    << "Qflag = " << qFlag << "\n";
        }
        
        std::cout << "----------------------------------\n";

        if (K > 32) {
            std::cout << "K = " << K << " is too large "
                      << "(currently we only consider K<=32)\n";
            exit(1);
        }
    }
};



inline int char_to_bits(char c) {

    int cvalue = -1;

    switch (c) {
        case 'A':
        case 'a':
            cvalue = 0;
            break;
        case 'C':
        case 'c':
            cvalue = 1;
            break;
        case 'G':
        case 'g':
            cvalue = 2;
            break;
        case 'T':
        case 't':
            cvalue = 3;
            break;
    }

    if (cvalue != -1) {
        return cvalue;
    } else {
        //std::cout << "non-ACGT character found: " << c << std::endl;
        return -1;
    }
}


inline bool toID(uint64_t& ID, char* addr, int len) {
    ID = 0;
    for (int i = 0; i < len; ++ i){
        int c = char_to_bits(addr[i]);
        if (c == -1) return false;
	ID  = (ID << 2 |  char_to_bits (addr[i]));
    }
    return true;
}


inline void prefix_sum (const iu64map_t& s2cnt, std::ofstream& oHandle){

    //int threshold = -1;
    std::vector<uint64_t> sum;
    //ivec_t sum;
    sum.reserve(s2cnt.size());

    uint64_t prefix = 0;

    for (iu64map_t::const_iterator it = s2cnt.begin(); it != s2cnt.end(); ++ it){
        prefix += (*it).second;
        sum.push_back(prefix);
    }

    int cnt = 0;

    if (!oHandle.good()){
        std::cout << "err prefix_sum: Can't open file to write ..."
                  << "output to screen: \n\n";


        std::cout << "\n Histogram\n";
        std::cout << "----------------------------\n\n";
        
        for (iu64map_t::const_iterator it = s2cnt.begin(); it != s2cnt.end(); ++ it){

            double quota = (double) sum[cnt]/sum.back();

            //if (err <= myPara.maxER) threshold = (*it).first;

            std::cout << (*it).first << "\t" << (*it).second
                      << "\t" << quota << "\n" ;
            cnt ++;
        }
        std::cout << "----------------------------\n";
    }
    else{
        
        oHandle << "----------------------------\n\n";
        
        for (iu64map_t::const_iterator it = s2cnt.begin(); it != s2cnt.end(); ++ it){

            double quota = (double) sum[cnt]/sum.back();

            //if (err <= myPara.maxER) threshold = (*it).first;

            oHandle << (*it).first << "\t" << (*it).second
                      << "\t" << quota << "\n" ;
            cnt ++;
        }
        oHandle << "----------------------------\n";

    }
}

inline double get_time() {
      timeval t;
      gettimeofday(&t, 0);
      return t.tv_sec + (0.000001 * t.tv_usec);
} // get_time


inline void print_time (const std::string& msg, double& timing){
    double cur_time = get_time();
    std::cout << msg << "(" << cur_time - timing << " secs)\n\n";
    timing = cur_time;
}

#endif	/* _UTIL_H */

